﻿
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Xml.Serialization;
using UAOOI.DataDiscovery.DiscoveryServices.Models;
using UAOOI.DataDiscovery.DiscoveryServices.UnitTest.TestData;

namespace UAOOI.DataDiscovery.DiscoveryServices.UnitTest
{
  [TestClass]
  public class DomainDescriptorUnitTest
{
    [TestMethod]
    public void GetRootDomainDescriptorTest()
    {
      DomainDescriptor _rootDomainDescriptor = DomainDescriptorFactory.GetRootDomainDescriptor();
      Uri _resolution = _rootDomainDescriptor.ResolveUri(m_ModelUri);
      string m_ExpectedFirsRoundUrl = "https://raw.githubusercontent.com/mpostol/OPC-UA-OOI/master/DataDiscovery/Tests/DiscoveryServices.UnitTest/TestData/root.zone/commsvr.com/DomainDescriptor.xml";
      Assert.AreEqual<string>(m_ExpectedFirsRoundUrl, _resolution.ToString());
      string _fn = "RootDomainDescriptor.xml";
      FileInfo _fi = new FileInfo($@"TestData\{_fn}");
      using (Stream _outputStream = _fi.Create())
      {
        XmlSerializer _serializer = new XmlSerializer(typeof(DomainDescriptor));
        _serializer.Serialize(_outputStream, _rootDomainDescriptor);
      }
      _fi.Refresh();
      Assert.IsTrue(_fi.Exists);
      Assert.IsTrue(_fi.Length > 0);
      DomainDescriptor _tc;
      using (Stream _descriptionStream = _fi.OpenRead())
      {
        XmlSerializer _serializer = new XmlSerializer(typeof(DomainDescriptor));
        _tc = (DomainDescriptor)_serializer.Deserialize(_descriptionStream);
        Assert.IsNotNull(_tc);
      }
      Assert.IsTrue(_tc.Description.Contains("Starting point"));
      Assert.AreEqual<RecordType>(RecordType.DomainDescriptor, _tc.NextStepRecordType);
      Assert.AreEqual<string>(@"https://raw.githubusercontent.com/mpostol/OPC-UA-OOI/master/DataDiscovery/Tests/DiscoveryServices.UnitTest/TestData/root.zone/#authority#/DomainDescriptor.xml", _tc.UrlPattern);
      _resolution = _tc.ResolveUri(m_ModelUri);
      Assert.AreEqual<string>(m_ExpectedFirsRoundUrl, _resolution.ToString());
    }
    [TestMethod]
    [DeploymentItem(@"TestData\", @"TestData\")]
    public void RootZoneDomainDescriptorTest()
    {
      DomainDescriptor _referenceDomainDescriptor = DomainDescriptorFactory.GetRootDomainDescriptor();
      AreEqualsDomainDescriptors(_referenceDomainDescriptor, @"root.zone\DomainDescriptor.xml");

      _referenceDomainDescriptor = DomainDescriptorFactory.Iteration1DomainDescriptor();
      AreEqualsDomainDescriptors(_referenceDomainDescriptor, @"root.zone\commsvr.com\DomainDescriptor.xml");

      _referenceDomainDescriptor = DomainDescriptorFactory.Iteration2DomainDescriptor();
      AreEqualsDomainDescriptors(_referenceDomainDescriptor, @"root.zone\commsvr.com\UA\Examples\BoilersSet\DomainDescriptor.xml");
    }
    [TestMethod]
    [DeploymentItem(@"TestData\", @"TestData\")]
    public void DomainDescriptorAutogeneratedFileTest()
    {
      FileInfo _fi = new FileInfo(@"TestData\DomainDescriptor.xml");
      Assert.IsTrue(_fi.Exists);
      DomainDescriptor _newDescription = null;
      using (Stream _descriptionStream = _fi.OpenRead())
      {
        XmlSerializer _serializer = new XmlSerializer(typeof(DomainDescriptor));
        _newDescription = (DomainDescriptor)_serializer.Deserialize(_descriptionStream);
      }
      Assert.IsNotNull(_newDescription);
      Assert.IsFalse(string.IsNullOrEmpty(_newDescription.Description));
      Assert.IsFalse(string.IsNullOrEmpty(_newDescription.UrlPattern));
    }

    //instrumentation
    Uri m_ModelUri = new Uri(@"http://commsvr.com/UA/Examples/BoilersSet");
    private static void AreEqualsDomainDescriptors(DomainDescriptor _rootDomainDescriptor, string fileName)
    {
      FileInfo _fi = new FileInfo($@"TestData\{fileName}");
      Assert.IsTrue(_fi.Exists);
      DomainDescriptor _tc;
      using (Stream _descriptionStream = _fi.OpenRead())
      {
        XmlSerializer _serializer = new XmlSerializer(typeof(DomainDescriptor));
        _tc = (DomainDescriptor)_serializer.Deserialize(_descriptionStream);
        Assert.IsNotNull(_tc);
      }
      Assert.AreEqual<string>(_rootDomainDescriptor.Description, _tc.Description);
      Assert.AreEqual<RecordType>(_rootDomainDescriptor.NextStepRecordType, _tc.NextStepRecordType);
      Assert.AreEqual<String>(_rootDomainDescriptor.UrlPattern, _tc.UrlPattern);
    }

  }
}


